"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Define: () => Define,
  DefineLocal: () => DefineLocal,
  DefineVariant: () => DefineVariant,
  Prop: () => Prop,
  Style: () => Style,
  default: () => src_default,
  hash: () => hash,
  pascalToDash: () => pascalToDash,
  t: () => t
});
module.exports = __toCommonJS(src_exports);

// src/syntax/assert.ts
var assert_exports = {};
__export(assert_exports, {
  and: () => and,
  anti: () => anti,
  falsy: () => falsy,
  inverseExpression: () => inverseExpression,
  isBinaryAssertion: () => isBinaryAssertion,
  isFalsy: () => isFalsy,
  isParenthesized: () => isParenthesized,
  ternary: () => ternary,
  truthy: () => truthy
});
var ASSERT_OP = /* @__PURE__ */ new Set([
  "in",
  "instanceof",
  "==",
  "===",
  "!=",
  "!==",
  ">",
  "<",
  ">=",
  "<="
]);
var INVERSE_OP = /* @__PURE__ */ new Map([
  ["==", "!="],
  ["===", "!=="],
  [">", "<="],
  ["<", ">="]
]);
for (const [a, b] of INVERSE_OP)
  INVERSE_OP.set(b, a);
function isParenthesized(node) {
  const { extra } = node;
  return extra ? extra.parenthesized === true : false;
}
function isBinaryAssertion(exp) {
  if (t.isBinaryExpression(exp) && ASSERT_OP.has(exp.operator))
    return true;
  return false;
}
function inverseExpression(exp) {
  const inverse = INVERSE_OP.get(exp.operator);
  if (inverse)
    return t.binaryExpression(inverse, exp.left, exp.right);
  throw new Error(`Can't invert binary comparison ${exp.operator}.`);
}
function isFalsy(exp) {
  return t.isUnaryExpression(exp, { operator: "!" });
}
function falsy(exp) {
  return isBinaryAssertion(exp) ? inverseExpression(exp) : t.unaryExpression("!", exp, true);
}
function and(a, b) {
  return t.logicalExpression("&&", a, b);
}
function anti(exp) {
  return isFalsy(exp) ? exp.argument : falsy(exp);
}
function truthy(a) {
  return isFalsy(a) || isBinaryAssertion(a) ? a : falsy(falsy(a));
}
function ternary(test, consequent, alternate) {
  return t.conditionalExpression(test, consequent, alternate);
}

// src/syntax/construct.ts
var construct_exports = {};
__export(construct_exports, {
  block: () => block,
  call: () => call,
  declare: () => declare,
  expression: () => expression,
  get: () => get,
  keyIdentifier: () => keyIdentifier,
  literal: () => literal,
  member: () => member,
  object: () => object,
  objectAssign: () => objectAssign,
  objectKeys: () => objectKeys,
  property: () => property,
  requires: () => requires,
  returns: () => returns,
  statement: () => statement,
  template: () => template
});
function expression(value) {
  try {
    return literal(value);
  } catch (err) {
    return value;
  }
}
function literal(value) {
  switch (typeof value) {
    case "string":
      return t.stringLiteral(value);
    case "number":
      return t.numericLiteral(value);
    case "boolean":
      return t.booleanLiteral(value);
    case "undefined":
      return t.identifier("undefined");
    case "object":
      if (value === null)
        return t.nullLiteral();
    default:
      throw new Error("Not a literal type");
  }
}
function keyIdentifier(name) {
  return /^[A-Za-z0-9$_]+$/.test(name) ? t.identifier(name) : t.stringLiteral(name);
}
function property(key, value) {
  let shorthand = false;
  if (typeof key == "string") {
    shorthand = t.isIdentifier(value, { name: key });
    key = keyIdentifier(key);
  }
  return t.objectProperty(key, value, false, shorthand);
}
function object(obj = {}) {
  let properties = [];
  if (Array.isArray(obj))
    properties = obj;
  else
    for (const [key, value] of Object.entries(obj))
      if (value)
        properties.push(property(key, value));
  return t.objectExpression(properties);
}
function get(object2, ...path) {
  if (object2 == "this")
    object2 = t.thisExpression();
  if (typeof object2 == "string")
    path = [...object2.split("."), ...path];
  for (const x of path) {
    let select;
    if (typeof x == "number")
      select = literal(x);
    else if (typeof x == "string")
      select = keyIdentifier(x);
    else if (t.isExpression(x))
      select = x;
    else
      throw new Error("Bad member id, only strings and numbers are allowed");
    object2 = typeof object2 == "object" ? member(object2, select) : select;
  }
  return object2;
}
function member(object2, property2) {
  return t.memberExpression(object2, property2, !t.isIdentifier(property2));
}
function call(callee, ...args) {
  if (typeof callee == "string")
    callee = get(callee);
  return t.callExpression(callee, args);
}
function requires(from) {
  return call("require", literal(from));
}
function returns(argument, parenthesized = false) {
  const statement2 = t.returnStatement(argument);
  statement2.extra = { parenthesized };
  return statement2;
}
function declare(kind, id, init) {
  return t.variableDeclaration(kind, [
    t.variableDeclarator(id, init || null)
  ]);
}
function objectAssign(...objects) {
  return call("Object.assign", ...objects);
}
function objectKeys(object2) {
  return call("Object.keys", object2);
}
function template(text) {
  return t.templateLiteral([
    t.templateElement({ raw: text, cooked: text }, false)
  ], []);
}
function statement(from) {
  return t.isExpression(from) ? t.expressionStatement(from) : from;
}
function block(...statements) {
  const stats = statements.map(statement);
  return t.blockStatement(stats);
}

// src/syntax/jsx.ts
var jsx_exports = {};
__export(jsx_exports, {
  HTML_TAGS: () => HTML_TAGS,
  SVG_TAGS: () => SVG_TAGS,
  jsxTag: () => jsxTag
});
var HTML_TAGS = [
  "a",
  "abbr",
  "address",
  "area",
  "article",
  "aside",
  "audio",
  "b",
  "base",
  "bdo",
  "bgsound",
  "blink",
  "blockquote",
  "br",
  "button",
  "canvas",
  "caption",
  "cite",
  "code",
  "col",
  "colgroup",
  "datalist",
  "dd",
  "del",
  "details",
  "dfn",
  "div",
  "dl",
  "dt",
  "em",
  "fieldset",
  "figure",
  "footer",
  "form",
  "h1",
  "h2",
  "h3",
  "h4",
  "h5",
  "h6",
  "header",
  "hgroup",
  "hr",
  "i",
  "img",
  "input",
  "ins",
  "kbd",
  "label",
  "legend",
  "li",
  "link",
  "map",
  "mark",
  "marquee",
  "meter",
  "multicol",
  "nav",
  "object",
  "ol",
  "optgroup",
  "option",
  "output",
  "p",
  "pre",
  "progress",
  "q",
  "samp",
  "section",
  "select",
  "small",
  "source",
  "span",
  "strong",
  "style",
  "sub",
  "sup",
  "table",
  "tbody",
  "td",
  "textarea",
  "tfoot",
  "th",
  "thead",
  "time",
  "tr",
  "ul",
  "var",
  "video",
  "wbr"
];
var SVG_TAGS = [
  "a",
  "altGlyph",
  "altGlyphDef",
  "altGlyphItem",
  "animate",
  "animateColor",
  "animateMotion",
  "animateTransform",
  "animation",
  "audio",
  "canvas",
  "circle",
  "clipPath",
  "color-profile",
  "cursor",
  "defs",
  "desc",
  "discard",
  "ellipse",
  "feBlend",
  "feColorMatrix",
  "feComponentTransfer",
  "feComposite",
  "feConvolveMatrix",
  "feDiffuseLighting",
  "feDisplacementMap",
  "feDistantLight",
  "feDropShadow",
  "feFlood",
  "feFuncA",
  "feFuncB",
  "feFuncG",
  "feFuncR",
  "feGaussianBlur",
  "feImage",
  "feMerge",
  "feMergeNode",
  "feMorphology",
  "feOffset",
  "fePointLight",
  "feSpecularLighting",
  "feSpotLight",
  "feTile",
  "feTurbulence",
  "filter",
  "font",
  "font-face",
  "font-face-format",
  "font-face-name",
  "font-face-src",
  "font-face-uri",
  "foreignObject",
  "g",
  "glyph",
  "glyphRef",
  "handler",
  "hkern",
  "iframe",
  "image",
  "line",
  "linearGradient",
  "listener",
  "marker",
  "mask",
  "metadata",
  "missing-glyph",
  "mpath",
  "path",
  "pattern",
  "polygon",
  "polyline",
  "prefetch",
  "radialGradient",
  "rect",
  "script",
  "set",
  "solidColor",
  "stop",
  "style",
  "svg",
  "switch",
  "symbol",
  "tbreak",
  "text",
  "textArea",
  "textPath",
  "title",
  "tref",
  "tspan",
  "unknown",
  "use",
  "video",
  "view",
  "vkern"
];
function jsxIdentifier(name) {
  return typeof name == "string" ? t.jsxIdentifier(name) : name;
}
function jsxTag(tag, props, children) {
  const type = jsxIdentifier(tag);
  const content = children.map(jsxContent);
  const contains = content.length > 0;
  const openingElement = t.jsxOpeningElement(type, props, !contains);
  const closingElement = contains ? t.jsxClosingElement(type) : null;
  return t.jsxElement(openingElement, closingElement, content, !contains);
}
function jsxContent(child) {
  if (t.isJSXElement(child))
    return child;
  if (t.isStringLiteral(child) && !/\{/.test(child.value))
    return t.jsxText(child.value);
  return t.jsxExpressionContainer(child);
}

// src/syntax/index.ts
var babel = {};
var t = {
  ...babel,
  ...assert_exports,
  ...construct_exports,
  ...jsx_exports
};

// src/handle/attributes.ts
var Attribute = class {
  constructor(name, value) {
    if (name)
      this.name = name;
    if (value !== void 0)
      this.value = value;
  }
  get expression() {
    const { value } = this;
    if (value && typeof value == "object" && "toExpression" in value)
      return value.toExpression() || t.expression();
    return t.expression(value);
  }
};
var Style = class extends Attribute {
  constructor(name, value, important) {
    if (Array.isArray(value))
      value = `${value[0]}(${value.slice(1).join(" ")})`;
    super(name, value);
    this.important = important || false;
    if (value === null || typeof value !== "object")
      this.invariant = true;
  }
};
var Prop = class extends Attribute {
};

// src/utility.ts
var m32 = Math.imul;
function hash(str = "", length = 3) {
  const x = 2246822507;
  const y = 3266489909;
  let h1 = 3735928559;
  let h2 = 1103547991;
  for (let i = 0, ch; i < str.length; i++) {
    ch = str.charCodeAt(i);
    h1 = m32(h1 ^ ch, 2654435761);
    h2 = m32(h2 ^ ch, 1597334677);
  }
  h1 = m32(h1 ^ h1 >>> 16, x) ^ m32(h2 ^ h2 >>> 13, y);
  h2 = m32(h2 ^ h2 >>> 16, x) ^ m32(h1 ^ h1 >>> 13, y);
  const out = 4294967296 * (2097151 & h2) + (h1 >>> 0);
  return out.toString(32).substring(0, length);
}
function ensureArray(a) {
  return Array.isArray(a) ? a : [a];
}
var ArrayStack = class extends Array {
  insert(x) {
    if (Array.isArray(this.top))
      this.top.push(x);
    else {
      this.top = [x];
      super.push(this.top);
    }
  }
  push(x) {
    this.top = x;
    return super.push(x);
  }
};
function pascalToDash(x) {
  return x.replace(/([A-Z]+)/g, "-$1").toLowerCase();
}

// src/generate/attributes.ts
var AttributeStack = class extends ArrayStack {
  constructor() {
    super(...arguments);
    this.exists = /* @__PURE__ */ new Set();
    this.invariant = /* @__PURE__ */ new Set();
  }
  insert(item, inline_only) {
    const { name } = item;
    if (name === void 0) {
      this.top = item;
      this.push(item);
      return;
    }
    if (this.exists.has(name))
      return;
    this.exists.add(name);
    if (item.invariant && !inline_only)
      this.invariant.add(item);
    else
      super.insert(item);
  }
  flatten() {
    if (!this.length)
      return;
    if (this.length == 1 && this[0] instanceof Style)
      return this[0].expression;
    const chunks = [];
    for (const item of this)
      if (item instanceof Style)
        chunks.push(t.spreadElement(item.expression));
      else
        chunks.push(...item.map(
          (style) => t.property(style.name, style.expression)
        ));
    return t.object(chunks);
  }
};

// src/generate/element.ts
var byPriority = (x, y) => x.priority - y.priority;
var Generator = class {
  constructor(element) {
    this.props = [];
    this.children = [];
    this.style = new AttributeStack();
    this.classList = /* @__PURE__ */ new Set();
    const { invariant } = this.style;
    this.context = element.context;
    this.tag = element.tagName;
    Array.from(element.includes).sort(byPriority).forEach((mod) => {
      if (this.inline_only && mod instanceof DefineVariant) {
        console.warn(`Cannot include CSS for ${mod.selector} with inline_only mode. Skipping.`);
        return;
      }
      const using_css = this.useClass(mod);
      for (const prop of mod.sequence)
        if (prop instanceof Style) {
          const { name, invariant: invariant2 } = prop;
          if (!name || invariant2 && using_css)
            continue;
          this.style.insert(prop);
        } else
          this.add(prop);
    });
    element.sequence.forEach(this.add, this);
    if (element instanceof ElementInline && invariant.size)
      element = new DefineLocal(element, invariant);
    if (element instanceof Define)
      this.useClass(element);
  }
  get inline_only() {
    return this.context.options.styleMode === "inline";
  }
  get info() {
    const { props, children } = this;
    const className = this.className();
    const stylesProp = this.style.flatten();
    if (className)
      props.push({ name: "className", value: className });
    if (stylesProp)
      props.push({ name: "style", value: stylesProp });
    return { props, children };
  }
  className() {
    const { classList, context } = this;
    const { file } = context;
    if (!classList.size)
      return;
    const selectors = [];
    let className = "";
    for (const item of classList)
      if (typeof item == "string")
        className += " " + item;
      else
        selectors.push(item);
    if (className)
      selectors.unshift(
        t.literal(className.slice(1))
      );
    if (selectors.length > 1) {
      const _use = file.ensure("$runtime", "classNames");
      return t.call(_use, ...selectors);
    }
    return selectors[0];
  }
  element(collapsable) {
    return this.context.file.element(this.info, this.tag, collapsable);
  }
  add(item) {
    if (item instanceof Style)
      this.style.insert(item, this.inline_only);
    else if (item instanceof Prop)
      this.applyProp(item);
    else if ("toExpression" in item) {
      const child = item.toExpression();
      if (child && (child.type !== "BooleanLiteral" || child.value !== false))
        this.children.push(child);
      if ("toClassName" in item) {
        const className = item.toClassName();
        if (className)
          this.classList.add(className);
      }
    } else if (t.isExpression(item))
      this.children.push(item);
  }
  applyProp(item) {
    const { name } = item;
    if (name === "style") {
      const style = new Style(false, item.expression);
      this.style.push(style);
      return;
    }
    if (name === "className") {
      let { value } = item;
      if (value && typeof value == "object") {
        if ("toExpression" in value)
          return;
        else if (t.isStringLiteral(value))
          ({ value } = value);
        else if (t.isExpression(value)) {
          this.classList.add(value);
          return;
        }
      }
      if (typeof value == "string")
        this.classList.add(value.trim());
      return;
    }
    this.props.push({ name, value: item.expression });
  }
  useClass(from) {
    if (this.inline_only)
      return false;
    from.setActive();
    if (from.isDeclared && from.isUsed) {
      const { context } = this;
      const { extractCss, cssModule } = context.options;
      let className = from.uid;
      if (extractCss && cssModule !== false)
        className = t.member(
          context.file.ensureUIDIdentifier("css"),
          t.identifier(className)
        );
      this.classList.add(className);
    }
    return true;
  }
};

// src/generate/es5.ts
function createElement(tag, properties = [], children = []) {
  const create = this.ensure("$pragma", "createElement", "create");
  if (!tag)
    tag = this.ensure("$pragma", "Fragment").name;
  const props = recombineProps(properties);
  const type = typeof tag === "string" ? /^[A-Z]/.test(tag) ? t.identifier(tag) : t.literal(tag) : stripJSX(tag);
  return t.call(create, type, props, ...children);
}
function recombineProps(props) {
  const propStack = new ArrayStack();
  if (props.length === 0)
    return t.object();
  for (const { name, value } of props)
    if (!name)
      propStack.push(value);
    else
      propStack.insert(
        t.property(name, value)
      );
  const properties = propStack.map(
    (chunk) => Array.isArray(chunk) ? t.object(chunk) : chunk
  );
  if (properties[0].type !== "ObjectExpression")
    properties.unshift(t.object());
  return properties.length > 1 ? t.objectAssign(...properties) : properties[0];
}
function stripJSX(exp) {
  switch (exp.type) {
    case "Identifier":
      return exp;
    case "JSXIdentifier":
      return t.identifier(exp.name);
    case "JSXMemberExpression":
      return t.member(stripJSX(exp.object), stripJSX(exp.property));
    default:
      throw new Error("Bad MemberExpression");
  }
}

// src/handle/object.ts
var AttributeBody = class {
  constructor(context, name) {
    this.sequence = [];
    this.statements = [];
    this.children = [];
    /** Other definitions applicable to this one. */
    this.includes = /* @__PURE__ */ new Set();
    context = Object.create(context);
    context.modifiers = Object.create(context.modifiers);
    if (name)
      context.name = this.name = name;
    this.context = context;
  }
  get uid() {
    return this.name + "_" + this.path();
  }
  path(salt) {
    let path = "";
    for (let ctx = this.context; ctx; ctx = ctx.parent)
      path += ctx.name;
    return hash(path + salt);
  }
  modify(from) {
    const applied = [];
    let modify = typeof from == "string" ? this.getModifier(from) : from;
    while (modify) {
      applied.push(modify);
      for (const use of [modify, ...modify.includes]) {
        this.includes.add(use);
        use.targets.add(this);
      }
      const context = modify.context.modifiers;
      Object.getOwnPropertyNames(context).map((name) => {
        this.setModifier(name, context[name]);
      });
      if (modify !== modify.then)
        modify = modify.then;
      else
        break;
    }
    return applied;
  }
  getModifier(name) {
    return this.context.modifiers[name];
  }
  setModifier(name, mod) {
    const next = this.context.modifiers[name];
    if (next === mod)
      return mod;
    if (next)
      mod.then = next;
    this.context.modifiers[name] = mod;
    return mod;
  }
  add(item) {
    this.sequence.push(item);
  }
  adopt(child) {
    const index = this.children.push(child);
    if ("context" in child)
      child.context.name = String(index);
    this.add(child);
  }
};

// src/handle/definition.ts
var ElementInline = class _ElementInline extends AttributeBody {
  adopt(child) {
    if (child instanceof _ElementInline)
      child.parent = this;
    super.adopt(child);
  }
  toExpression() {
    return new Generator(this).element(this.selfClosing);
  }
};
var Define = class extends AttributeBody {
  constructor(context, name) {
    super(context, name);
    this.priority = 1;
    /** Targets which this modifier applies to. */
    this.targets = /* @__PURE__ */ new Set();
    /** Modifiers based upon this one. */
    this.dependant = /* @__PURE__ */ new Set();
    this.context.define = this;
    if (name && /^[A-Z]/.test(name))
      this.priority = 2;
  }
  get selector() {
    return [`.${this.uid}`];
  }
  get isUsed() {
    if (this.context.declared.has(this))
      return true;
    if (this.targets.size)
      return true;
    for (const child of this.dependant)
      if (child.isUsed || child instanceof DefineVariant)
        return true;
  }
  get isDeclared() {
    const queue = /* @__PURE__ */ new Set([this]);
    for (const x of queue) {
      if (x.hasStyle(true))
        return true;
      x.dependant.forEach((x2) => queue.add(x2));
    }
  }
  use(mod) {
    this.includes.add(mod);
    if (mod instanceof DefineVariant)
      this.dependant.add(mod);
  }
  provide(define) {
    define.container = this;
    this.setModifier(define.name, define);
  }
  toExpression(maybeProps) {
    const { info } = new Generator(this);
    if (maybeProps && info.children.length === 0)
      return recombineProps(info.props);
    return this.context.file.container(info);
  }
  addStyle(name, value) {
    this.add(new Style(name, value));
  }
  setActive(withPriority) {
    if (!this.hasStyle(true))
      return;
    if (withPriority > this.priority)
      this.priority = withPriority;
    this.context.declared.add(this);
  }
  hasStyle(arg) {
    return this.sequence.find((style) => {
      if (style instanceof Style)
        return typeof arg == "string" ? arg == style.name : !arg || style.invariant;
    });
  }
};
var DefineLocal = class extends Define {
  constructor(parent, styles) {
    super(parent.context);
    this.parent = parent;
    this.sequence = Array.from(styles);
    this.priority = 2;
  }
  get uid() {
    return this.parent.uid;
  }
};
var DefineVariant = class extends Define {
  constructor(parent, suffix, priority) {
    super(parent.context);
    this.parent = parent;
    this.suffix = suffix;
    this.priority = priority;
    this.within = parent.within;
    parent.use(this);
  }
  get selector() {
    let { suffix } = this;
    if (typeof suffix == "string")
      suffix = [suffix];
    return suffix.map((select) => this.parent.selector + select);
  }
  get uid() {
    return this.parent.uid;
  }
  get isUsed() {
    return false;
  }
  provide(define) {
    this.dependant.add(define);
    this.parent.provide(define);
    define.within = this;
    define.priority = this.priority;
  }
};

// src/handle/macros.ts
function forwardProp(...propNames) {
  const target = this.target;
  const exec = this.body.getAncestry().find((x) => x.isFunction());
  if (!exec)
    throw new Error("No function-component found in hierarchy");
  const { scope } = exec;
  const properties = getProps(exec);
  for (const key of propNames)
    if (key === "ref")
      forwardRef(target, exec);
    else {
      const _local = uniqueWithin(scope, key);
      const property2 = t.property(key, _local);
      const prop = new Prop(key, _local);
      properties.unshift(property2);
      target.add(prop);
    }
}
function forwardRef(target, component) {
  const { node } = component;
  const { file } = target.context;
  if (t.isFunctionDeclaration(node))
    node.type = "FunctionExpression";
  const _ref = uniqueWithin(component.scope, "ref");
  const _forwardRef = file.ensure("$pragma", "forwardRef");
  const _wrapped = t.call(_forwardRef, node);
  component.pushContainer("params", _ref);
  component.replaceWith(_wrapped);
  target.add(new Prop("ref", _ref));
}
function uniqueWithin(scope, name) {
  return scope.hasBinding(name) ? scope.generateUidIdentifier(name) : t.identifier(name);
}
function getProps(exec) {
  const { node } = exec;
  let props = node.params[0];
  if (!t.isObjectPattern(props)) {
    const existing = props;
    props = t.objectPattern([]);
    if (!existing)
      node.params[0] = props;
    else if (t.isIdentifier(existing)) {
      const { body } = node.body;
      for (const stat of body) {
        if (!t.isVariableDeclaration(stat))
          break;
        for (const { id, init } of stat.declarations)
          if (t.isIdentifier(init, { name: existing.name }) && t.isObjectPattern(id))
            return id.properties;
      }
      body.unshift(
        t.declare("const", props, existing)
      );
    }
  }
  return props.properties;
}
function applyAlso(...names) {
  const { target } = this;
  for (const name of names)
    if (typeof name == "string") {
      const mod = target.getModifier(name);
      if (mod)
        target.use(mod);
    }
}
function setPriority(priority) {
  this.target.priority = priority;
}
var builtIn = {
  forward: forwardProp,
  priority: setPriority,
  use: applyAlso
};

// src/parse/entry.ts
function getName(path) {
  let encounteredReturn;
  while (path)
    switch (path.type) {
      case "VariableDeclarator": {
        const { id } = path.node;
        return t.isIdentifier(id) ? id.name : path.parentPath.node.kind;
      }
      case "AssignmentExpression":
      case "AssignmentPattern": {
        const { left } = path.node;
        return t.isIdentifier(left) ? left.name : "assignment";
      }
      case "FunctionDeclaration":
        return path.node.id.name;
      case "ExportDefaultDeclaration":
        return getLocalFilename(path.hub);
      case "ArrowFunctionExpression": {
        path = path.parentPath;
        continue;
      }
      case "ReturnStatement": {
        if (encounteredReturn)
          return "return";
        encounteredReturn = path;
        const ancestry = path.getAncestry();
        const within = ancestry.find((x) => x.isFunction());
        if (!within)
          throw new Error("wat");
        const { node } = within;
        if ("id" in node && node.id)
          return node.id.name;
        if (t.isObjectMethod(node)) {
          path = within.getAncestry()[2];
          continue;
        }
        if (t.isClassMethod(node)) {
          if (node.key.type !== "Identifier")
            return "ClassMethod";
          if (node.key.name == "render") {
            const owner = within.parentPath.parentPath;
            if (owner.node.id)
              return owner.node.id.name;
            path = owner.parentPath;
            continue;
          } else
            return node.key.name;
        }
        path = within.parentPath;
        continue;
      }
      case "ObjectProperty": {
        const { key } = path.node;
        return t.isIdentifier(key) ? key.name : t.isStringLiteral(key) ? key.value : "property";
      }
      default:
        return "element";
    }
  return "element";
}
function getLocalFilename(hub) {
  try {
    const { basename, dirname, sep: separator } = require("path");
    const url = hub.file.opts.filename;
    const [base] = basename(url).split(".");
    if (base !== "index")
      return base;
    return dirname(url).split(separator).pop();
  } catch (err) {
    return "File";
  }
}

// src/syntax/uid.ts
function ensureUID(path, name = "temp") {
  const { scope } = path;
  let uid = name = name.replace(/^_+/, "").replace(/[0-9]+$/g, "");
  for (let i = 2; scope.hasBinding(uid) || scope.hasGlobal(uid) || scope.hasReference(uid); i++) {
    uid = name + i;
  }
  const program = scope.getProgramParent();
  program.references[uid] = true;
  program.uids[uid] = true;
  return uid;
}

// src/generate/jsx.ts
var OUTPUT_NODE = /* @__PURE__ */ new WeakSet();
function createElement2(tag, properties = [], content = []) {
  if (!tag)
    tag = this.ensure("$pragma", "Fragment").name;
  const props = properties.map(({ name, value }) => {
    if (typeof name !== "string")
      return t.jsxSpreadAttribute(value);
    const jsxValue = t.isStringLiteral(value) ? value.value === "true" ? null : value : t.jsxExpressionContainer(value);
    return t.jsxAttribute(
      t.jsxIdentifier(name),
      jsxValue
    );
  });
  this.ensure("$pragma", "default", "React");
  const element = t.jsxTag(tag, props, content);
  OUTPUT_NODE.add(element);
  return element;
}

// src/scope.ts
var FileManager = class {
  constructor(path, options) {
    this.path = path;
    this.declaredUIDIdentifiers = {};
    this.imports = {};
    this.importIndices = {};
    this.options = options;
    this.createElement = options.output === "js" ? createElement : createElement2;
  }
  static create(options, path) {
    const { externals, output } = options;
    const Type = externals == "require" ? RequireManager : externals == "import" ? ImportManager : output == "js" ? RequireManager : ImportManager;
    return new Type(path, options);
  }
  element(src, tagName, collapsable) {
    const { children, props } = src;
    const tag = tagName || (props.length || !collapsable || !children.length ? "div" : null);
    if (!tag && children.length === 1)
      return children[0];
    return this.createElement(tag, props, children);
  }
  container(src, key) {
    let { children } = src;
    if (src.props.length)
      children = [this.element(src)];
    if (children.length > 1 || key) {
      const props = key && [{ name: "key", value: key }];
      return this.createElement(null, props, children);
    }
    return children[0] || t.literal(false);
  }
  replaceAlias(value) {
    if (!value.startsWith("$"))
      return value;
    const name = value.slice(1);
    return this.options[name];
  }
  ensureUIDIdentifier(name) {
    const exist = this.declaredUIDIdentifiers;
    return exist[name] || (exist[name] = t.identifier(ensureUID(this.path, name)));
  }
  close() {
    if (this.options.externals === false)
      return;
    Object.entries(this.imports).forEach(([name, external]) => {
      if (external.exists)
        return;
      const importStatement = this.createImport(name);
      if (importStatement) {
        const index = this.importIndices[name];
        this.path.node.body.splice(index, 0, importStatement);
      }
    });
  }
};
var ImportManager = class extends FileManager {
  constructor() {
    super(...arguments);
    this.imports = {};
  }
  ensure(from, name, alt = name) {
    from = this.replaceAlias(from);
    let uid;
    const list = this.ensureImported(from).items;
    if (name == "default") {
      const [spec] = list;
      if (t.isImportDefaultSpecifier(spec))
        return spec.local;
      uid = typeof alt == "string" ? this.ensureUIDIdentifier(alt) : alt;
      list.unshift(t.importDefaultSpecifier(uid));
      return uid;
    }
    for (const spec of list)
      if ("imported" in spec) {
        const { imported } = spec;
        if (imported.type == "Identifier" && imported.name == name) {
          uid = t.identifier(spec.local.name);
          break;
        }
      }
    if (!uid) {
      uid = typeof alt == "object" ? alt : this.ensureUIDIdentifier(alt || name);
      list.push(
        t.importSpecifier(uid, t.identifier(name))
      );
    }
    return uid;
  }
  ensureImported(name) {
    const { imports } = this;
    name = this.replaceAlias(name);
    if (imports[name])
      return imports[name];
    for (const stat of this.path.node.body)
      if (t.isImportDeclaration(stat) && stat.source.value == name)
        return imports[name] = {
          exists: true,
          items: stat.specifiers
        };
    return imports[name] = {
      exists: false,
      items: []
    };
  }
  createImport(name) {
    const list = this.imports[name].items;
    if (list.length)
      return t.importDeclaration(list, t.literal(name));
  }
};
var RequireManager = class extends FileManager {
  constructor() {
    super(...arguments);
    this.imports = {};
    this.importTargets = {};
  }
  ensure(from, name, alt = name) {
    const source = this.ensureImported(from).items;
    for (const { key, value } of source)
      if (t.isIdentifier(value) && t.isIdentifier(key, { name }))
        return value;
    const ref = typeof alt == "string" ? this.ensureUIDIdentifier(alt) : alt;
    source.push(t.property(name, ref));
    return ref;
  }
  ensureImported(name) {
    const { imports, importTargets } = this;
    name = this.replaceAlias(name);
    if (name in imports)
      return imports[name];
    let target;
    let list;
    for (let i = 0, stat; stat = this.path.node.body[i]; i++)
      if (t.isVariableDeclaration(stat))
        target = requireResultFrom(name, stat);
    if (t.isObjectPattern(target))
      list = imports[name] = {
        exists: true,
        items: target.properties
      };
    else {
      list = imports[name] = { items: [] };
      if (target && target.type === "Identifier")
        importTargets[name] = target;
    }
    return list;
  }
  createImport(name) {
    const list = this.imports[name].items;
    if (list.length) {
      const target = this.importTargets[name] || t.requires(name);
      return t.declare("const", t.objectPattern(list), target);
    }
  }
};
function requireResultFrom(name, statement2) {
  for (const { init, id } of statement2.declarations)
    if (t.isCallExpression(init)) {
      const { callee, arguments: [arg] } = init;
      if (t.isIdentifier(callee, { name: "require" }) && t.isStringLiteral(arg, { value: name }))
        return id;
    }
}

// src/context.ts
var DEFAULTS = {
  env: "web",
  styleMode: "compile",
  runtime: "@expressive/css",
  pragma: "react",
  output: "js",
  macros: []
};
var Context = class {
  constructor(path, state) {
    this.declared = /* @__PURE__ */ new Set();
    this.modifiers = {};
    var _a;
    const { module: module2, macros } = state.opts;
    path.data = { context: this };
    this.options = { ...DEFAULTS, ...state.opts };
    this.macros = Object.assign({}, builtIn, ...macros || []);
    this.file = FileManager.create(this.options, path);
    this.name = hash(state.filename);
    this.filename = state.filename;
    this.define = new Define(this, this.name);
    this.module = module2 && (typeof module2 == "string" ? module2 : ((_a = state.file.opts.configFile) == null ? void 0 : _a.name) || true);
  }
  get parent() {
    return Object.getPrototypeOf(this);
  }
  getHandler(named, ignoreOwn = false) {
    let context = this;
    if (ignoreOwn)
      for (let found; !found; ) {
        found = context.handlers.has(named);
        context = context.parent;
      }
    const [key, ...path] = named.split(".");
    let handler = this.macros[key];
    for (const key2 of path)
      handler = handler[key2];
    return handler;
  }
  static get(path, create) {
    var _a;
    while (path = path.parentPath) {
      const scope = (_a = path.data) == null ? void 0 : _a.context;
      if (scope)
        return scope;
      if (!path.isBlockStatement() || !create)
        continue;
      const parentContext = this.get(path);
      if (!parentContext)
        throw new Error("well that's awkward.");
      const name = getName(path.parentPath);
      const define = new Define(parentContext, name);
      const { context } = define;
      if (path.node)
        path.data = { context };
      context.name = getName(path.parentPath);
      return context;
    }
    throw new Error("Scope not found!");
  }
};

// src/errors.ts
var Status = {
  currentFile: void 0
};
function ParseErrors(register) {
  const Errors = {};
  for (const error in register) {
    const message = [];
    for (const segment of register[error].split(/\{(?=\d+\})/)) {
      const ammend = /(\d+)\}(.*)/.exec(segment);
      if (ammend)
        message.push(parseInt(ammend[1]), ammend[2]);
      else
        message.push(segment);
    }
    Errors[error] = (node, ...args) => {
      let quote = "";
      if ("node" in node)
        node = node.node;
      for (const slice of message)
        quote += typeof slice == "string" ? slice : args[slice - 1];
      return Status.currentFile.buildCodeFrameError(node, quote);
    };
  }
  return Errors;
}

// src/generate/iterate.ts
var Oops = ParseErrors({
  BadForOfAssignment: 'Assignment of variable left of "of" must be Identifier or Destruture',
  BadForInAssignment: "Left of ForInStatement must be an Identifier here!"
});
function forElement(node, define) {
  const { statements } = define;
  const { file } = define.context;
  const output = define.toExpression();
  if (!output)
    return;
  const accumulator = file.ensureUIDIdentifier("add");
  const collect = file.ensure("$runtime", "collect");
  let body = t.statement(
    t.call(accumulator, output)
  );
  if (statements.length)
    body = t.block(...statements, body);
  node.body = body;
  return t.call(
    collect,
    t.arrowFunctionExpression([accumulator], t.block(node))
  );
}
function forXElement(node, define) {
  let { left, right, key } = getReferences(node);
  key = ensureKeyProp(define, key);
  let body = define.toExpression();
  if (!body)
    return;
  if (define.statements.length)
    body = t.block(...define.statements, t.returns(body));
  if (t.isForOfStatement(node)) {
    const params = key ? [left, key] : [left];
    return t.call(
      t.get(right, "map"),
      t.arrowFunctionExpression(params, body)
    );
  }
  return t.call(
    t.get(t.objectKeys(right), "map"),
    t.arrowFunctionExpression([left], body)
  );
}
function getReferences(node) {
  let { left, right } = node;
  let key;
  if (t.isVariableDeclaration(left))
    left = left.declarations[0].id;
  switch (left.type) {
    case "Identifier":
    case "ObjectPattern":
    case "ArrayPattern":
      break;
    default:
      throw Oops.BadForOfAssignment(left);
  }
  if (t.isBinaryExpression(right, { operator: "in" })) {
    key = right.left;
    right = right.right;
  }
  if (t.isForInStatement(node))
    if (t.isIdentifier(left))
      key = left;
    else
      throw Oops.BadForInAssignment(left);
  return { left, right, key };
}
function ensureKeyProp(from, used) {
  let { sequence, children } = from;
  const props = sequence.filter((x) => x instanceof Prop);
  for (const x of props)
    if (x.name === "key")
      return;
  if (children.length == 1 && props.length == 0) {
    const element = children[0];
    if (element instanceof ElementInline) {
      for (const x of element.sequence)
        if (x instanceof Prop && x.name === "key")
          return;
      sequence = element.sequence;
    }
  }
  if (!used)
    used = from.context.file.ensureUIDIdentifier("i");
  sequence.push(new Prop("key", used));
  return used;
}

// src/handle/iterate.ts
var ComponentFor = class {
  constructor(path, parent) {
    const name = path.type.replace("Statement", "Loop");
    const element = new Define(parent.context, name);
    parse(element, path, "body");
    this.definition = element;
    this.node = path.node;
    parent.adopt(this);
  }
  toExpression() {
    const { node, definition } = this;
    return t.isForStatement(node) ? forElement(node, definition) : forXElement(node, definition);
  }
};

// src/handle/switch.ts
var conseqentsExist = /* @__PURE__ */ new Map();
var DefineConsequent = class extends Define {
  constructor(parent) {
    super(parent.context);
    this.parent = parent;
    this.within = parent.within;
    this.priority = 5;
    this.rank = (conseqentsExist.get(this) || 0) + 1;
    this.name = this.parent.name;
    conseqentsExist.set(this, this.rank);
  }
  get isDeclared() {
    return false;
  }
  get uid() {
    return this.name + "_" + this.path(this.rank);
  }
  provide(define) {
    this.dependant.add(define);
    this.parent.provide(define);
    define.within = this;
    define.priority = this.priority;
  }
};
var ComponentIf = class {
  constructor() {
    this.forks = [];
  }
  setup(context, path) {
    do {
      if (!path.isIfStatement()) {
        this.include(context, path);
        break;
      }
      const test = path.get("test");
      const consequent = path.get("consequent");
      this.include(context, consequent, test);
      path = path.get("alternate");
    } while (path.type);
  }
  toExpression() {
    return this.reduce((cond) => {
      if (cond.children.length)
        return cond.toExpression();
    });
  }
  toClassName() {
    return this.reduce((cond) => {
      const segments = [];
      for (const mod of [cond, ...cond.includes]) {
        mod.setActive(cond.priority);
        if (mod.isUsed)
          segments.push(mod.uid);
      }
      if (segments.length)
        return t.literal(segments.join(" "));
    });
  }
  include(context, body, test) {
    const { forks } = this;
    if (body.isIfStatement())
      throw new Error("Nested if statements are not supported.");
    if (body.isBlockStatement()) {
      const inner = ensureArray(body.get("body"));
      if (inner.length == 1)
        body = inner[0];
    }
    const define = new DefineConsequent(context.define);
    define.context.name = String(forks.length);
    parse(define, body);
    forks.push([define, test && test.node]);
    return define;
  }
  reduce(predicate) {
    const forks = this.forks.slice().reverse();
    let sum;
    for (const [cond, test] of forks) {
      const product = predicate(cond);
      if (sum && test)
        sum = product ? t.ternary(test, product, sum) : t.and(t.anti(test), sum);
      else if (product)
        sum = test ? t.and(t.truthy(test), product) : product;
    }
    return sum;
  }
  specify(test) {
    if (!test)
      return false;
    if (t.isFalsy(test) && t.isIdentifier(test.argument))
      return `not_${test.argument.name}`;
    if (t.isIdentifier(test))
      return test.name;
  }
};

// src/parse/jsx.ts
var Oops2 = ParseErrors({
  InvalidPropValue: "Can only consume an expression or string literal as value here.",
  UnhandledChild: "Can't parse JSX child of type {1}",
  JSXMemberExpression: "Member Expression is not supported!",
  NonJSXIdentifier: "Cannot handle non-identifier!"
});
function addElementFromJSX(parent, path) {
  let target = parent;
  if (path.isJSXElement()) {
    const tag = path.get("openingElement").get("name");
    if (!tag.isJSXIdentifier({ name: "this" })) {
      const child = new ElementInline(target.context);
      applyTagName(child, tag.node);
      target.adopt(child);
      target = child;
    }
  }
  parseJSX(target, path);
}
function parseJSX(into, element) {
  const queue = [[into, element]];
  for (const [element2, path] of queue) {
    const children = path.get("children");
    if (path.isJSXElement()) {
      const { name: tag, selfClosing } = path.node.openingElement;
      const attributes = path.get("openingElement").get("attributes");
      if (element2 instanceof ElementInline)
        element2.selfClosing = selfClosing;
      applyTagName(element2, tag);
      for (const attribute of attributes)
        applyAttribute(element2, attribute, queue);
    }
    for (const path2 of children)
      applyChild(element2, path2, queue);
  }
}
function applyChild(element, path, queue) {
  if (path.isJSXElement()) {
    const child = new ElementInline(element.context);
    element.adopt(child);
    queue.push([child, path]);
  } else if (path.isJSXText()) {
    const { value } = path.node;
    if (/^\n+ *$/.test(value))
      return;
    const text = value.replace(/ +/g, " ").replace(/\n\s*/g, "");
    element.adopt(t.literal(text));
  } else if (path.isJSXExpressionContainer()) {
    const { expression: expression2 } = path.node;
    if (!t.isJSXEmptyExpression(expression2))
      element.adopt(path.node.expression);
  } else
    Oops2.UnhandledChild(path, path.type);
}
function applyTagName(element, tag) {
  let name;
  if (t.isJSXMemberExpression(tag)) {
    name = tag.property.name;
    element.tagName = tag;
  } else if (t.isJSXIdentifier(tag)) {
    name = tag.name;
    if (name === "this")
      return;
    element.name = name;
    let explicit = /^[A-Z]/.test(name);
    if (/^(html|svg)-.+/.test(name)) {
      name = name.slice(5);
      explicit = true;
    }
    if (HTML_TAGS.includes(name))
      explicit = true;
    else if (SVG_TAGS.includes(name)) {
      element.name = name;
      let within = element;
      while (within instanceof ElementInline)
        if (within.name === "svg") {
          explicit = true;
          break;
        } else {
          within = within.parent;
          continue;
        }
    }
    if (explicit)
      element.tagName = name;
  } else
    throw Oops2.NonJSXIdentifier(tag);
  element.modify(name);
}
function applyAttribute(parent, attr, queue) {
  let name;
  let value;
  if (attr.isJSXSpreadAttribute()) {
    name = false;
    value = attr.node.argument;
  } else {
    const expression2 = attr.node.value;
    name = attr.node.name.name;
    if (!expression2) {
      const applied = parent.modify(name);
      if (!applied.length)
        value = t.booleanLiteral(true);
      else {
        if (/^[A-Z]/.test(parent.name))
          for (const define of applied)
            define.priority = 3;
        return;
      }
    } else
      switch (expression2.type) {
        case "JSXExpressionContainer":
          value = expression2.expression;
          break;
        case "StringLiteral":
          if (name == "src" && /^\.\//.test(expression2.value))
            value = t.requires(expression2.value);
          else
            value = expression2;
          break;
        default:
          throw Oops2.InvalidPropValue(expression2);
      }
  }
  if (value.type == "JSXElement") {
    const path = attr.get("value.expression");
    const child = new ElementInline(parent.context);
    const prop = new Prop(name, child);
    applyTagName(child, path.node.openingElement.name);
    parent.add(prop);
    queue.push([child, path]);
    return child;
  }
  parent.add(
    new Prop(name, value || null)
  );
}

// src/parse/arguments.ts
var Oops3 = ParseErrors({
  UnaryUseless: "Unary operator here doesn't do anything",
  HexNoNegative: `Hexadecimal numbers are converted into colors (#FFF) so negative sign doesn't mean anything here.
Parenthesize the number if you really need "-{1}" for some reason...`,
  ArrowNotImplemented: "Arrow Function not supported yet",
  ArgumentSpread: "Cannot parse argument spreads for modifier handlers",
  UnknownArgument: "Unknown argument while parsing for modifier.",
  MustBeIdentifier: "Only Identifers allowed here! Call expression must reference another modifier.",
  ModiferCantParse: "Illegal value in modifier",
  ElseNotSupported: "An else statement in an if modifier is not yet supported"
});
var types = {
  Identifier,
  StringLiteral,
  TemplateLiteral,
  UnaryExpression,
  BooleanLiteral,
  NumericLiteral,
  NullLiteral,
  BinaryExpression,
  SequenceExpression,
  CallExpression,
  ArrowFunctionExpression,
  IfStatement,
  LabeledStatement,
  BlockStatement
};
function parse2(element) {
  if (t.isExpressionStatement(element))
    element = element.expression;
  return [].concat(
    t.isExpression(element) ? Expression(element) : Extract(element)
  );
}
function Expression(element, childKey) {
  if (childKey)
    element = element[childKey];
  if (t.isParenthesized(element))
    return element;
  return Extract(element);
}
function Extract(element) {
  if (element.type in types)
    return types[element.type](element);
  throw Oops3.UnknownArgument(element);
}
function Identifier({ name }) {
  if (name.startsWith("$")) {
    name = pascalToDash(name.slice(1));
    return `var(--${name})`;
  }
  return name;
}
function StringLiteral(e) {
  return e.value;
}
function TemplateLiteral(temp) {
  if (temp.quasis.length == 1)
    return temp.quasis[0].value.raw;
  return temp;
}
function UnaryExpression(e) {
  const { argument, operator } = e;
  if (operator == "-" && t.isNumericLiteral(argument))
    return NumericLiteral(argument, true);
  if (operator == "!" && t.isIdentifier(argument, { name: "important" }))
    return "!important";
  throw Oops3.UnaryUseless(e);
}
function BooleanLiteral(bool) {
  return bool.value;
}
function NumericLiteral(number, negative) {
  let { extra: { rawValue, raw } } = number;
  if (t.isParenthesized(number) || !/^0x/.test(raw)) {
    if (raw.indexOf(".") > 0)
      return negative ? "-" + raw : raw;
    return negative ? -rawValue : rawValue;
  }
  if (negative)
    throw Oops3.HexNoNegative(number, rawValue);
  return HEXColor(raw);
}
function NullLiteral() {
  return null;
}
function BinaryExpression(binary) {
  const { left, right, operator } = binary;
  if (operator == "-" && t.isIdentifier(left) && t.isIdentifier(right, { start: left.end + 1 }))
    return left.name + "-" + right.name;
  return [
    operator,
    Expression(binary, "left"),
    Expression(binary, "right")
  ];
}
function SequenceExpression(sequence) {
  return sequence.expressions.map((x) => Expression(x));
}
var CSS_UNITS = /* @__PURE__ */ new Set([
  "ch",
  "cm",
  "em",
  "ex",
  "in",
  "mm",
  "pc",
  "pt",
  "px",
  "rem",
  "vh",
  "vmax",
  "vmin",
  "vw"
]);
function CallExpression(e) {
  const callee = e.callee;
  const args = [];
  if (callee.type !== "Identifier")
    throw Oops3.MustBeIdentifier(callee);
  for (const item of e.arguments) {
    if (t.isExpression(item))
      args.push(Expression(item));
    else if (t.isSpreadElement(item))
      throw Oops3.ArgumentSpread(item);
    else
      throw Oops3.UnknownArgument(item);
  }
  const { name } = callee;
  if (CSS_UNITS.has(name))
    return args.map((x) => String(x) + name).join(" ");
  return pascalToDash(callee.name) + `(${args.join(", ")})`;
}
function ArrowFunctionExpression(e) {
  throw Oops3.ArrowNotImplemented(e);
}
function IfStatement(statement2) {
  const alt = statement2.alternate;
  const test = statement2.test;
  const body = statement2.consequent;
  if (alt)
    throw Oops3.ElseNotSupported(test);
  const info = body.type == "BlockStatement" ? BlockStatement(body) : body.type == "LabeledStatement" ? LabeledStatement(body) : body.type == "ExpressionStatement" ? Expression(body.expression) : {};
  return {
    test: Expression(test),
    ...info
  };
}
function LabeledStatement(stat) {
  return {
    [stat.label.name]: parse2(stat.body)
  };
}
function BlockStatement(statement2) {
  const map = {};
  for (const item of statement2.body)
    if (t.isLabeledStatement(item))
      map[item.label.name] = parse2(item.body);
    else if (item.type !== "IfStatement")
      throw Oops3.ModiferCantParse(statement2);
  return map;
}
function HEXColor(raw) {
  raw = raw.substring(2);
  if (raw.length == 1)
    raw = "000" + raw;
  else if (raw.length == 2)
    raw = "000000" + raw;
  if (raw.length % 4 == 0) {
    let decimal = [];
    if (raw.length == 4)
      decimal = Array.from(raw).map((x) => parseInt(x + x, 16));
    else
      for (let i = 0; i < 4; i++)
        decimal.push(
          parseInt(raw.slice(i * 2, i * 2 + 2), 16)
        );
    decimal[3] = (decimal[3] / 255).toFixed(3);
    return `rgba(${decimal.join(",")})`;
  }
  return "#" + raw;
}

// src/parse/labels.ts
var Oops4 = ParseErrors({
  BadModifierName: "Modifier name cannot start with _ symbol!",
  InlineModeNoVariants: "Cannot attach a CSS variant while styleMode is set to inline.",
  DollarSignDeprecated: "Dollar-sign macros are deprecated. Did you mean to use a namespace?"
});
function handleDefine(target, path) {
  const { context } = target;
  let name = getName2(path);
  let body = path.get("body");
  if (body.isBlockStatement()) {
    const mod = new Define(context, name);
    target.provide(mod);
    parse(mod, body);
    return;
  }
  while (body.isLabeledStatement()) {
    name = `${name}.${body.node.label.name}`;
    body = body.get("body");
  }
  if (body.isIfStatement())
    name = `${name}.if`;
  const args = parse2(body.node);
  const include = context.getHandler("default") || defaultModifier;
  let important = false;
  if (args[args.length - 1] == "!important") {
    important = true;
    args.pop();
  }
  const queue = [{ name, args, body }];
  while (queue.length) {
    const { name: name2, body: body2, args: args2 } = queue.pop();
    const modifier = new ModifyDelegate(target, name2, body2);
    const transform = context.getHandler(name2) || include;
    const output = transform.apply(modifier, args2);
    if (!output)
      continue;
    if (Array.isArray(output)) {
      defaultModifier.apply(modifier, output);
      continue;
    }
    if (typeof output != "object")
      throw new Error("Invalid modifier output.");
    for (const key in output) {
      let args3 = output[key];
      if (args3 === void 0)
        continue;
      if (!Array.isArray(args3))
        args3 = [args3];
      if (important)
        args3.push("!important");
      if (key === name2)
        defaultModifier.apply(modifier, args3);
      else
        queue.push({ name: key, args: args3 });
    }
  }
}
function getName2(path) {
  const { name } = path.get("label").node;
  if (name.startsWith("_"))
    throw Oops4.BadModifierName(path);
  if (name.startsWith("$"))
    return name.replace(/^\$/, "--");
  return name;
}
var defaultModifier = function() {
  this.addStyle(this.name, ...arguments);
};
var ModifyDelegate = class {
  constructor(target, name, body) {
    this.target = target;
    this.name = name;
    this.body = body;
    this.context = target.context;
  }
  addStyle(name, ...args) {
    const parsed = args.map((arg) => arg.value || (arg.requires ? t.requires(arg.requires) : arg));
    const output = parsed.length == 1 || typeof parsed[0] == "object" ? parsed[0] : Array.from(parsed).join(" ");
    this.target.addStyle(name, output);
  }
  setContingent(select, priority, usingBody) {
    if (!usingBody)
      usingBody = this.body;
    const mod = new DefineVariant(this.target, select, priority || 1);
    parse(mod, usingBody);
    return mod;
  }
};

// src/parse/block.ts
var Oops5 = ParseErrors({
  PropNotIdentifier: "Assignment must be identifier name of a prop."
});
function parse(target, block2, key) {
  if (key)
    block2 = block2.get(key);
  const body = block2.isBlockStatement() ? ensureArray(block2.get("body")) : [block2];
  for (const item of body) {
    switch (item.type) {
      case "LabeledStatement":
        handleDefine(target, item);
        break;
      case "IfStatement":
        handleIfStatement(target, item);
        break;
      case "ForInStatement":
      case "ForOfStatement":
      case "ForStatement":
        new ComponentFor(item, target);
        break;
      case "ExpressionStatement": {
        const expr = item.get("expression");
        if (expr.isAssignmentExpression({ operator: "=" })) {
          handlePropAssignment(target, expr);
          break;
        }
        if (expr.isJSXElement() || expr.isJSXFragment()) {
          OUTPUT_NODE.add(expr.node);
          addElementFromJSX(target, expr);
        }
      }
      default:
        continue;
    }
    item.remove();
  }
}
function handlePropAssignment(target, expr) {
  const { left, right } = expr.node;
  if (!t.isIdentifier(left))
    throw Oops5.PropNotIdentifier(left);
  const prop = new Prop(left.name, right);
  target.add(prop);
}
function handleIfStatement(target, path) {
  const test = path.node.test;
  if (t.isStringLiteral(test)) {
    let select = test.value;
    if (/^\w+$/.test(select))
      select = `.${select}`;
    const body = path.get("consequent");
    const mod = new DefineVariant(target, [select], 5);
    parse(mod, body);
    target.use(mod);
    return;
  }
  const item = new ComponentIf();
  item.setup(target.context, path);
  target.adopt(item);
}

// src/index.ts
var src_default = (babel2) => {
  Object.assign(t, babel2.types);
  return {
    manipulateOptions(options, parse3) {
      parse3.plugins.push("jsx");
    },
    visitor: {
      Program,
      JSXElement,
      JSXFragment: JSXElement
    }
  };
};
var Program = {
  enter(path, state) {
    Status.currentFile = state.file;
    state.context = new Context(path, state);
  },
  exit(_path, state) {
    state.context.file.close();
  }
};
var NODE_HANDLED = /* @__PURE__ */ new WeakSet();
var JSXElement = {
  enter(path) {
    if (OUTPUT_NODE.has(path.node))
      return;
    if (!path.parentPath.isExpressionStatement()) {
      const context2 = Context.get(path, true);
      const target = new ElementInline(context2);
      parseJSX(target, path);
      path.replaceWith(
        target.toExpression()
      );
      return;
    }
    const ancestry = path.getAncestry();
    let containerFunction;
    for (const item of ancestry) {
      if (item.isLabeledStatement())
        break;
      if (item.isFunction()) {
        containerFunction = item;
        break;
      }
    }
    if (!containerFunction)
      return;
    NODE_HANDLED.add(path.node);
    const functionNode = containerFunction.node;
    const block2 = containerFunction.get("body");
    const context = Context.get(path, true);
    const { define } = context;
    parse(define, block2);
    const output = define.toExpression();
    if (!output)
      return;
    const { body } = block2.node;
    if (path.parentPath.node == body[0] || !body[0])
      functionNode.body = output;
    else
      block2.node.body.push(t.returns(output));
    NODE_HANDLED.add(block2.node);
  },
  exit(path) {
    if (NODE_HANDLED.has(path.node))
      path.remove();
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Define,
  DefineLocal,
  DefineVariant,
  Prop,
  Style,
  hash,
  pascalToDash,
  t
});
